@@ -26,21 +26,13 @@ __license__ = "GPLv3"
 __collaborator__ = "Jean-Michel Garant"
 
 import os
-os.environ['PYTHONHASHSEED'] = '0'
 import numpy as np
 import random as rn
 import tensorflow as tf
-
-rn.seed(1)
-np.random.seed(1)
-tf.random.set_seed(1)
-
-#The tensorflow version is 2.1.0
-#The scikit-learn version is 0.21.3
+import warnings
 import sys
 import multiprocessing as mp
 import argparse
-import os
 import pysam
 from matplotlib import pyplot as plt
 from tqdm import tqdm
@@ -48,6 +40,9 @@ from itertools import repeat
 from pickle import dump, load
 import statistics
 import pandas as pd
+import gzip
+import bz2
+from collections import defaultdict
 try:
     from sklearn.preprocessing import StandardScaler
     from sklearn.metrics import roc_curve
@@ -150,7 +145,19 @@ def model_plot(fpr_keras, tpr_keras,output,history):
     plt.close('all')
 
 
-def window_mutation(window_list, bam_file,mq,reference,depth,nine_mer):
+def openfile(file):
+    '''
+    Opens a file
+    '''
+    if file.endswith('.gz'):
+        opened_file = gzip.open(file,'rt')
+    elif file.endswith('bz') or file.endswith('bz2'):
+        opened_file = bz2.open(file,'rt')
+    else:
+        opened_file = open(file,'rt')
+    return opened_file
+
+def window_mutation(window_list, bam_file,mq,reference,depth):
     """
     Function to calculate mutation frequenies and qualities (features). The
     features then used by prediction or extraction mudules to be used for
@@ -167,25 +174,18 @@ def window_mutation(window_list, bam_file,mq,reference,depth,nine_mer):
         except:
             raise Exception("Cannot load reference file.")
         try:
-            try:
-                window_seq= fasta.fetch(reference=chrom,
-                                        start=windowstart,
-                                        end=windowend)
-            except:
-                window_seq= fasta.fetch(reference=chrom[3:],
-                                        start=windowstart,
-                                        end=windowend)
+            window_seq= fasta.fetch(reference=chrom,
+                                    start=windowstart,
+                                    end=windowend)
         except: # Window is not in the reference
+            warnings.warn("Reference sequence for {}:{}-{} "
+                          "was not found in the reference file"
+                          "".format(chrom,windowstart,windowend))
             continue
         window_seq=str(window_seq.upper())
-        try:
-            pileupcolumns= bam.pileup(chrom, windowstart, windowend,
-                                      truncate= True,min_base_quality = 0,
-                                      min_mapping_quality = mq)
-        except:
-            pileupcolumns= bam.pileup(chrom[3:], windowstart, windowend,
-                                      truncate= True,min_base_quality = 0,
-                                      min_mapping_quality = mq)
+        pileupcolumns= bam.pileup(chrom, windowstart, windowend,
+                                  truncate= True,min_base_quality = 0,
+                                  min_mapping_quality = mq)
         refbase_index= 0
         del_list= []
         mis_list= []
@@ -222,58 +222,32 @@ def window_mutation(window_list, bam_file,mq,reference,depth,nine_mer):
                 ins_list.append(insertion/coverage)
                 qual_list.append(quality)
                 cov_list.append(coverage)
-        if len(cov_list) == 9:# to check is list is empty and ignor partial in case no reads were mapped
+        if len(cov_list) == 9:# to check if list is empty and ignor partial in case no reads were mapped
             #converting to 5-mer
-            if not nine_mer:
-                for index in range(0,5):
-                    kmer_dumy= []
-                    for base in window_seq[index: index + 5]:
-                        kmer_dumy.extend(encoder[base])
-                    mean_quals= statistics.mean(qual_list[index: index + 5])
-                    median_quals= statistics.median(qual_list[index: index + 5])
-                    std_quals= statistics.stdev(qual_list[index: index + 5])
-                    mean_mis= statistics.mean(mis_list[index: index + 5])
-                    std_mis= statistics.stdev(mis_list[index: index + 5])
-                    mean_dels= statistics.mean(del_list[index: index + 5])
-                    std_del= statistics.stdev(del_list[index: index + 5])
-                    mean_ins= statistics.mean(ins_list[index: index + 5])
-                    std_ins= statistics.stdev(ins_list[index: index + 5])
-                    out_frequencies= (cov_list[index: index + 5]+
-                                      kmer_dumy+
-                                      qual_list[index: index + 5]+
-                                      mis_list[index: index + 5]+
-                                      del_list[index: index + 5]+
-                                      ins_list[index: index + 5]+
-                                      [mean_quals, median_quals, std_quals,
-                                       mean_mis, std_mis, mean_dels, std_del,
-                                       mean_ins,std_ins])
-                    key= (chrom, windowstart+index,windowstart+index + 5,
-                          base_pos, window_seq[index: index + 5])
-                    freq_dict[key]= (window , out_frequencies)
-            else:
+            for index in range(0,5):
                 kmer_dumy= []
-                for base in window_seq:
+                for base in window_seq[index: index + 5]:
                     kmer_dumy.extend(encoder[base])
-                mean_quals= statistics.mean(qual_list)
-                median_quals= statistics.median(qual_list)
-                std_quals= statistics.stdev(qual_list)
-                mean_mis= statistics.mean(mis_list)
-                std_mis= statistics.stdev(mis_list)
-                mean_dels= statistics.mean(del_list)
-                std_del= statistics.stdev(del_list)
-                mean_ins= statistics.mean(ins_list)
-                std_ins= statistics.stdev(ins_list)
-                out_frequencies= (cov_list+
+                mean_quals= statistics.mean(qual_list[index: index + 5])
+                median_quals= statistics.median(qual_list[index: index + 5])
+                std_quals= statistics.stdev(qual_list[index: index + 5])
+                mean_mis= statistics.mean(mis_list[index: index + 5])
+                std_mis= statistics.stdev(mis_list[index: index + 5])
+                mean_dels= statistics.mean(del_list[index: index + 5])
+                std_del= statistics.stdev(del_list[index: index + 5])
+                mean_ins= statistics.mean(ins_list[index: index + 5])
+                std_ins= statistics.stdev(ins_list[index: index + 5])
+                out_frequencies= (cov_list[index: index + 5]+
                                   kmer_dumy+
-                                  qual_list+
-                                  mis_list+
-                                  del_list+
-                                  ins_list+
+                                  qual_list[index: index + 5]+
+                                  mis_list[index: index + 5]+
+                                  del_list[index: index + 5]+
+                                  ins_list[index: index + 5]+
                                   [mean_quals, median_quals, std_quals,
                                    mean_mis, std_mis, mean_dels, std_del,
-                                   mean_ins, std_ins])
-                key= (chrom, windowstart, windowend,
-                      base_pos, window_seq)
+                                   mean_ins,std_ins])
+                key= (chrom, windowstart+index,windowstart+index + 5,
+                      base_pos, window_seq[index: index + 5])
                 freq_dict[key]= (window , out_frequencies)
         else:
             continue
@@ -291,14 +265,10 @@ def main_extraction(args):
     mod_status= args.mod_status
     window_list=list()
     feed_list= list()
-    bed_file= os.path.abspath(args.input)
-    if not bed_file.endswith('.bed') and not bed_file.endswith('.vcf'):
-        raise TypeError("Input file is not a vcf or bed file.")
-    bed= open(bed_file)
-    if bed_file.endswith('.bed'):
-        next(bed)
+    vcf_file= os.path.abspath(args.input)
+    vcf= openfile(vcf_file)
     all_lines= []
-    for line in bed:
+    for line in vcf:
         if line.startswith('#'):
             continue
         all_lines.append(1)
@@ -306,30 +276,20 @@ def main_extraction(args):
                          for x in range(0, len(all_lines), chunk)]
     all_lines = [all_lines[x:x+threads]
                          for x in range(0, len(all_lines), threads)]
-    bed.close()
-    bed= open(bed_file)
-    if bed_file.endswith('.bed'):
-        next(bed)
+    vcf.close()
+    vcf= openfile(vcf_file)
     with tqdm(total=len(all_lines),desc="Processing: ",
                       bar_format="{l_bar}{bar} [ Estimated time left:"
                       " {remaining} ]") as pbar:
-        for line in bed:
+        for line in vcf:
             if line.startswith('#'):
                 continue
             line=line.rstrip().split('\t')
             chrom= line[0]
-            if 'chr' not in chrom.lower():
-                chrom= 'chr'+chrom
-            if bed_file.endswith('.bed'):
-                windowinfo= (chrom,
-                             int(line[1])-4,
-                             int(line[1])+5,
-                             int(line[1]))
-            else:
-                windowinfo= (chrom,
-                             int(line[1])-4-1,
-                             int(line[1])+5-1,
-                             int(line[1])-1)
+            windowinfo= (chrom, 
+                         int(line[1])-4-1,
+                         int(line[1])+5-1,
+                         int(line[1])-1)
             if windowinfo not in window_list:
                 window_list.append(windowinfo)
                 if len(window_list) > chunk:
@@ -342,16 +302,15 @@ def main_extraction(args):
                                      repeat(bam_file),
                                      repeat(mq),
                                      repeat(reference),
-                                     repeat(args.depth),
-                                     repeat(args.nine_mer))))
+                                     repeat(args.depth))))
                     p.close()
                     p.join()
                     for freq_dict in results:
                         if freq_dict is not None:
                             for key,val in freq_dict.items():
-                                print(','.join(map(str,key)) + ',' +
+                                sys.stdout.write(','.join(map(str,key)) + ',' + 
                                       ','.join(map(str,val[1])) +
-                                      ',' + str(mod_status))
+                                      ',' + str(mod_status)+'\n')
                     feed_list = []
                     pbar.update(1)
         else:
@@ -363,19 +322,18 @@ def main_extraction(args):
                                  repeat(bam_file),
                                  repeat(mq),
                                  repeat(reference),
-                                 repeat(args.depth),
-                                 repeat(args.nine_mer))))
+                                 repeat(args.depth))))
                 p.close()
                 p.join()
                 for freq_dict in results:
                     if freq_dict is not None:
                         for key,val in freq_dict.items():
-                            print(','.join(map(str,key)) + ',' +
-                                  ','.join(map(str,val[1])) +
-                                  ',' + str(mod_status))
-                pbar.update(1)
+                            sys.stdout.write(','.join(map(str,key)) + ',' + 
+                                  ','.join(map(str,val[1])) + 
+                                  ',' + str(mod_status)+'\n')
+                pbar.update(1)                    
                 feed_list = []
-    bed.close()
+    vcf.close()
     sys.stderr.write("Job Finished\n")
 
 def main_prediction(args):
@@ -392,25 +350,26 @@ def main_prediction(args):
     threads= args.threads
     chunk= args.chunk_size
     reference= os.path.abspath(args.reference)
-    bed_file= os.path.abspath(args.input)
-    if not bed_file.endswith('.bed') and not bed_file.endswith('.vcf'):
-        raise TypeError("Input file is not a vcf or bed file.")
-    bed= open(bed_file)
-    if bed_file.endswith('.bed'):
-        next(bed)
+    vcf_file= os.path.abspath(args.input)
+    output= os.path.abspath(args.output)
+    if (not os.path.isfile(output+"_Predictions.vcf") and not 
+        os.path.isfile(output+"_Weighted_Qualities.vcf")):
+        out_preds= open(output+"_Predictions.vcf", 'w')
+        out_ready= open(output+"_Weighted_Qualities.vcf",'w')
+    else:
+        raise FileExistsError("Selected output files already exists.")
+    vcf= openfile(vcf_file)
     all_lines= []
-    for line in bed:
+    for line in vcf:
         if line.startswith('#'):
-            continue
+            out_ready.write(line)
         all_lines.append(1)
-    bed.close()
+    vcf.close()
     all_lines= [all_lines[x:x+chunk]
                          for x in range(0, len(all_lines), chunk)]
     all_lines= [all_lines[x:x+threads]
                          for x in range(0, len(all_lines), threads)]
-    bed= open(bed_file)
-    if bed_file.endswith('.bed'):
-        next(bed)
+    vcf= openfile(vcf_file)
     feed_list= list()
     windowinfo= dict()
     model_input= []
@@ -418,24 +377,15 @@ def main_prediction(args):
     with tqdm(total=len(all_lines),desc="Processing: ",
                       bar_format="{l_bar}{bar} [ Estimated time left: {remaining} ]"
                       ) as pbar:
-        for line in bed:
+        for line in vcf:
             if line.startswith('#'):
-                print(line.rstrip())
                 continue
             line_list= line.rstrip().split('\t')
             chrom= line_list[0]
-            if 'chr' not in chrom.lower():
-                chrom= 'chr'+chrom
-            if bed_file.endswith('.bed'):
-                windowinfo[(chrom,
-                            int(line_list[1])-4,
-                            int(line_list[1])+5,
-                            int(line_list[1]))]= line.rstrip()
-            else:
-                windowinfo[(chrom,
-                            int(line_list[1])-4-1,
-                            int(line_list[1])+5-1,
-                            int(line_list[1])-1)]= line.rstrip()
+            windowinfo[(chrom,
+                        int(line_list[1])-4-1,
+                        int(line_list[1])+5-1,
+                        int(line_list[1])-1)]= line.rstrip()
             if len(list(windowinfo.keys())) == (chunk * threads):
                 feed_list= [list(windowinfo.keys())[x:x+chunk]
                             for x in range(0, len(list(windowinfo.keys())),
@@ -446,21 +396,15 @@ def main_prediction(args):
                                  repeat(bam_file),
                                  repeat(mq),
                                  repeat(reference),
-                                 repeat(args.depth),
-                                 repeat(args.nine_mer))))
+                                 repeat(args.depth))))
                 p.close()
                 p.join()
                 for freq_dict in results:
                     if freq_dict is not None:
                         for key,val in freq_dict.items():
-                            if not args.nine_mer:
-                                coverage= int(statistics.mean(val[1][0:5]))
-                                info_input.append((key,coverage,val[0]))
-                                model_input.append(val[1][5:])
-                            else:
-                                coverage= int(statistics.mean(val[1][0:9]))
-                                info_input.append((key,coverage,val[0]))
-                                model_input.append(val[1][9:])
+                            coverage= int(statistics.mean(val[1][0:5]))
+                            info_input.append((key,coverage,val[0]))
+                            model_input.append(val[1][5:])
                 model_input= np.asarray(model_input, dtype= np.float64)
                 model_input= scaler.transform(model_input)
                 predictions= model.predict(model_input)
@@ -469,9 +413,9 @@ def main_prediction(args):
                     cov= info_input[i][1]
                     key_windoinfo= info_input[i][2]
                     pred= float(predictions[i])
-                    print(windowinfo[key_windoinfo] + '\t' +
+                    out_preds.write(windowinfo[key_windoinfo] + '\t' +
                           '\t'.join(map(str,key)) + '\t' +
-                          str(cov) + '\t' + str(pred))
+                          str(cov) + '\t' + str(pred)+'\n')
                 model_input= []
                 info_input= []
                 windowinfo= dict()
@@ -488,21 +432,15 @@ def main_prediction(args):
                                  repeat(bam_file),
                                  repeat(mq),
                                  repeat(reference),
-                                 repeat(args.depth),
-                                 repeat(args.nine_mer))))
+                                 repeat(args.depth))))
                 p.close()
                 p.join()
                 for freq_dict in results:
                     if freq_dict is not None:
                         for key,val in freq_dict.items():
-                            if not args.nine_mer:
-                                coverage= int(statistics.mean(val[1][0:5]))
-                                info_input.append((key,coverage,val[0]))
-                                model_input.append(val[1][5:])
-                            else:
-                                coverage= int(statistics.mean(val[1][0:9]))
-                                info_input.append((key,coverage,val[0]))
-                                model_input.append(val[1][9:])
+                            coverage= int(statistics.mean(val[1][0:5]))
+                            info_input.append((key,coverage,val[0]))
+                            model_input.append(val[1][5:])
                 model_input= np.asarray(model_input, dtype= np.float64)
                 model_input= scaler.transform(model_input)
                 predictions= model.predict(model_input)
@@ -511,13 +449,25 @@ def main_prediction(args):
                     cov= info_input[i][1]
                     key_windoinfo= info_input[i][2]
                     pred= float(predictions[i])
-                    print(windowinfo[key_windoinfo] + '\t' +
+                    out_preds.write(windowinfo[key_windoinfo] + '\t' +
                           '\t'.join(map(str,key)) + '\t' +
-                          str(cov)+'\t'+str(pred))
+                          str(cov)+'\t'+str(pred)+'\n')
                 windowinfo= dict()
                 feed_list = []
                 pbar.update(1)
-    bed.close()
+    vcf.close()
+    out_preds.close()
+    ready_dict= defaultdict(list)
+    with openfile(output+"_Predictions.vcf") as pred:
+        for line in pred:
+            line=line.rstrip().split('\t')
+            ready_dict[tuple(line[0:-7])].append(float(line[-1]))
+    for key,val in ready_dict.items():
+        if key[5].replace('.','',1).isdigit():
+            weighted_qual= float(key[5]) * statistics.mean(val)
+            out_write= list(key[0:5])+[round(weighted_qual,4)]+list(key[6:])
+            out_ready.write('\t'.join(map(str,out_write))+'\n')
+    out_ready.close()
     sys.stderr.write("Job Finished\n")
 
 def main_train(args):
@@ -527,22 +477,15 @@ def main_train(args):
     train_data= os.path.abspath(args.train)
     test_data= os.path.abspath(args.test)
     output= os.path.abspath(args.out_dir)
-    nine_mer= args.nine_mer
     epoch_num= args.epochs
     batch_num= args.batch_size
 
     train = pd.read_csv(train_data, header= None)
     test= pd.read_csv(test_data, header= None)
-    if not nine_mer:
-        X_train= train.iloc[:, 10:-1].values
-        y_train = train.iloc[:, -1].values
-        test_val= test.iloc[:, 10:-1].values
-        test_label= test.iloc[:, -1].values
-    else:
-        X_train= train.iloc[:, 14:-1].values
-        y_train = train.iloc[:, -1].values
-        test_val= test.iloc[:, 14:-1].values
-        test_label= test.iloc[:, -1].values
+    X_train= train.iloc[:, 10:-1].values
+    y_train = train.iloc[:, -1].values
+    test_val= test.iloc[:, 10:-1].values
+    test_label= test.iloc[:, -1].values
     inputdim= X_train.shape[1]
     # Feature Scaling
     sc = StandardScaler()
@@ -608,15 +551,14 @@ def prediction_parser(subparsers):
     """
     sub_prediction = subparsers.add_parser(
         "prediction",
-        description=("Predict based on a model.\n"))
-    sp_input = sub_prediction.add_argument_group("INPUT\n.")
+        help="predict using a model",
+        description=("Predict based on a model."))
+    sp_input = sub_prediction.add_argument_group("required arguments")
     sp_input.add_argument("--input", "-i", action="store", type=str,
                           required=True,help="The path to the input "
                           "vcf or bed file. NOTE. Files must end with "
                           ".bed or .vcf. vcf files are 1-based and beds "
                           "are zero-based")
-    sp_input.add_argument("--model_file", "-mf", type=str, action="store",
-                          required=True,help="Pathn to the trained model.")
     sp_input.add_argument("--bam", "-b", action="store", type=str,
                           required=True,help="The path to the alignment bam "
                           "file")
@@ -624,6 +566,18 @@ def prediction_parser(subparsers):
                           required=True, default=None, help="The path to the "
                           "reference file. File must be indexed by samtools "
                           "faidx.")
+    sp_input.add_argument("--output", "-o", action="store", type=str,
+                          required=True, default=None, help="The path to the "
+                          "output directory and prefix for output file.")
+    sp_input = sub_prediction.add_argument_group("optional arguments")
+    sp_input.add_argument("--model_file", "-mf", type=str, action="store",
+                          required=False,
+                          default=os.path.join(os.path.dirname(
+                                                   os.path.realpath(__file__)),
+                                               "model",
+                                               "NA12878_20FC_model.h5"),
+                          help="Path to the trained model. Default is "
+                               "NA12878_20FC_model.h5")
     sp_input.add_argument("--mappingQuality", "-mq", action="store", type=int,
                           default= 0,required=False,help="Cutt off for filtering "
                           "out low quality mapped reads from bam. Default is 0")
@@ -635,14 +589,11 @@ def prediction_parser(subparsers):
                           required=False, help="if you want to only do for a "
                           "region or chromosom You must insert region like "
                           "this chr1 or chr1:1000-100000.")
-    sp_input.add_argument("--nine_mer", "-nm", action="store_true",
-                          required=False, help="Prediction for 9-mer. "
-                          "Default is five mer")
     sp_input.add_argument("--threads", "-t", action="store", type=int,
                           required=False, default=4, help="Number of threads. "
                           "Default is 4.")
     sp_input.add_argument("--chunk_size", "-cs", action="store", type=int,
-                          required=False, default=100, help="Chunk size ."
+                          required=False, default=100, help="Chunk size. "
                           "Default is 100.")
     sub_prediction.set_defaults(func=main_prediction)
 
@@ -652,17 +603,16 @@ def extraction_parser(subparsers):
     """
     sub_extraction = subparsers.add_parser(
         "extraction",
-        description=("Extract mutation frequencicies in 5-mer window.\n"))
-    se_input = sub_extraction.add_argument_group("INPUT\n.")
+        help="extract features",
+        description=("Extract mutation frequencicies in 5-mer window."))
+    se_input = sub_extraction.add_argument_group("required arguments")
     se_input.add_argument("--input", "-i", action="store", type=str,
-                          required=True, help="The path to the input vcf or "
-                          "bed file. NOTE. Files must end with .bed or .vcf. "
-                          "vcf files are 1-based and beds are zero-based")
-    se_input.add_argument("--mod_status", "-ms", type=int,action="store",
+                          required=True, help="The path to the input vcf")
+    se_input.add_argument("--status", "-s", type=int,action="store",
                           required=True, help= "0 or 1. If you are extracting "
                           "frequencies to train a model, give the"
-                        " modification status for your bed file either it is "
-                        "modified (1) or unmodified (0) regions.")
+                        " status for your vcf file either it is "
+                        "true SNVs (1) or false calls (0).")
     se_input.add_argument("--bam", "-b", action="store", type=str,
                           required= True, help= "The path to the alignment "
                           "bam file")
@@ -670,6 +620,7 @@ def extraction_parser(subparsers):
                           required=True, default=None, help="The path to the "
                           "reference file. File must be indexed by samtools "
                           "faidx")
+    se_input = sub_extraction.add_argument_group("optional arguments")
     se_input.add_argument("--mappingQuality", "-mq", action="store", type=int,
                           default= 0,required=False,help="Cutt off for filtering "
                           "out low quality mapped reads from bam. Default is 0")
@@ -680,9 +631,6 @@ def extraction_parser(subparsers):
                           required=False, help= "if you want to only do for a "
                           "region or chromosom, you must insert region like "
                           "this chr1 or chr1:1000-100000.")
-    se_input.add_argument("--nine_mer", "-nm", action="store_true",
-                          required=False, help="Extraction for 9-mer. "
-                          "Default is five mer")
     se_input.add_argument("--threads", "-t", action="store", type=int,
                           required=False, default= 4, help="Number of threads")
     se_input.add_argument("--chunk_size", "-cs", action="store", type=int,
@@ -697,30 +645,28 @@ def train_parser(subparsers):
     """
     sub_train = subparsers.add_parser(
         "train",
-        description=("train a new model.\n"))
-    st_input = sub_train.add_argument_group("INPUT")
+        help="train a new model",
+        description=("train a new model"))
+    st_input = sub_train.add_argument_group("required arguments")
     st_input.add_argument("--train", "-tr", action="store", type=str,
                           required= True, help= "The path to the shuffled and "
-                          "ready file for training\n")
+                          "ready file for training")
     st_input.add_argument("--test", "-te", action="store", type=str,
                           required= True, help= "The path to the shuffled and "
-                          "ready file for testing.\n")
+                          "ready file for testing.")
+    st_input.add_argument("--out_dir", "-o", action="store", type=str,
+                          required= True, help= "Output directory and prefix "
+                          "for saving the model and figures")
+    st_input = sub_train.add_argument_group("optional arguments")
     st_input.add_argument("--epochs", "-e", action="store", type=int,
                           default= 100, required= False, help= "Number of "
                           "epochs. Default is 100")
     st_input.add_argument("--batch_size", "-batch", action="store", type=int,
                           required= False, default= 400, help= "batch size for "
                           "model training. Default is 400.")
-    st_input.add_argument("--out_dir", "-o", action="store", type=str,
-                          required= True, help= "Output directory and prefix "
-                          "for saving the model and figures")
     st_input.add_argument("--plot", "-plt", action= "store_true",
                           required= False, help= "Select this option if you "
                           "wish to output training plots.")
-    st_input.add_argument("--nine_mer", "-nm", action= "store_true",
-                          required= False, help="Training for 9-mer. "
-                          "Default is five mer")
-
     sub_train.set_defaults(func=main_train)
 
 def main():
@@ -728,14 +674,10 @@ def main():
     Docstring placeholder.
     """
     parser = argparse.ArgumentParser(
-        formatter_class=argparse.RawTextHelpFormatter,
-        prog="snv_voter: Voting for snvs\n",
-        description=("\t\t\tModules\n"
-                     "extraction: Extract Features\n"
-                     "prediction: Predict using model.\n"
-                     "train: train a new model.\n"))
-    subparsers = parser.add_subparsers(title="\t\t\tModules\n",
-                                       help='use -h/--help for help')
+#        formatter_class=argparse.RawTextHelpFormatter, #  It's bad habit
+        prog="snvoter",
+        description=("SNVoter: Voting for SNVs"))
+    subparsers = parser.add_subparsers(title="Modules")
     extraction_parser(subparsers)
     prediction_parser(subparsers)
     train_parser(subparsers)
@@ -747,4 +689,4 @@ def main():
 
 
 if __name__ == '__main__':
-    main()
+    main()
\ No newline at end of file
